"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getHighlighter = getHighlighter;
exports.getLoadedLanguages = getLoadedLanguages;
exports.getSafeLanguage = getSafeLanguage;
exports.getThemeContext = getThemeContext;
exports.isValidLanguage = isValidLanguage;
var _core = require("@shikijs/core");
var _engineJavascript = require("@shikijs/engine-javascript");
var _vscodeTextmate = require("@shikijs/vscode-textmate");
var _css = _interopRequireDefault(require("shiki/dist/langs/css.mjs"));
var _html = _interopRequireDefault(require("shiki/dist/langs/html.mjs"));
var _javascript = _interopRequireDefault(require("shiki/dist/langs/javascript.mjs"));
var _json = _interopRequireDefault(require("shiki/dist/langs/json.mjs"));
var _markdown = _interopRequireDefault(require("shiki/dist/langs/markdown.mjs"));
var _php = _interopRequireDefault(require("shiki/dist/langs/php.mjs"));
var _tsx = _interopRequireDefault(require("shiki/dist/langs/tsx.mjs"));
var _typescript = _interopRequireDefault(require("shiki/dist/langs/typescript.mjs"));
var _githubDark = _interopRequireDefault(require("shiki/dist/themes/github-dark.mjs"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable no-console */

// Import languages

const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
// Process language grammars
function processLanguage(lang) {
  if (!lang) {
    throw new Error('Invalid language definition');
  }

  // Create language registration
  const registration = {
    name: lang.name,
    scopeName: lang.scopeName,
    aliases: lang.aliases,
    embeddedLangs: lang.embeddedLanguages,
    repository: lang.repository,
    patterns: lang.patterns
  };
  log(`Processed language ${registration.name}`, {
    name: registration.name,
    scopeName: registration.scopeName,
    aliases: registration.aliases,
    embeddedLangs: registration.embeddedLangs,
    repository: registration.repository,
    patterns: registration.patterns
  });
  return registration;
}

// Define raw languages
const rawLanguages = {
  typescript: _typescript.default,
  javascript: _javascript.default,
  json: _json.default,
  html: _html.default,
  css: _css.default,
  markdown: _markdown.default,
  tsx: _tsx.default,
  php: _php.default,
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{
        match: '.*',
        name: 'text.plain'
      }]
    }
  }
};

// Process languages into proper registrations
const languageRegistrations = new Map();
for (const [key, lang] of Object.entries(rawLanguages)) {
  try {
    const registration = processLanguage(lang);
    languageRegistrations.set(key, registration);
    // Also register by scopeName for lookups
    languageRegistrations.set(registration.scopeName, registration);
    // Register aliases
    registration.aliases?.forEach(alias => {
      languageRegistrations.set(alias, registration);
    });
  } catch (error) {
    log(`Failed to process language ${key}:`, error);
  }
}
function normalizeTheme(rawTheme) {
  const theme = {
    ...rawTheme
  };
  theme.type ||= 'dark';
  theme.settings ||= [];
  theme.colorReplacements ||= {};
  if (theme.tokenColors && !theme.settings) {
    theme.settings = theme.tokenColors;
    delete theme.tokenColors;
  }
  theme.settings ||= [];
  theme.bg ||= theme.type === 'dark' ? '#1e1e1e' : '#ffffff';
  theme.fg ||= theme.type === 'dark' ? '#bbbbbb' : '#333333';
  if (!theme.settings[0] || !theme.settings[0].settings || theme.settings[0].scope) {
    theme.settings.unshift({
      settings: {
        foreground: theme.fg,
        background: theme.bg
      }
    });
  }
  return theme;
}
let highlighterInstance = null;
const themeCache = new Map();
function getThemeContext(theme) {
  const themeId = typeof theme === 'string' ? theme : theme.name || 'default';
  if (!themeCache.has(themeId)) {
    const normalizedTheme = normalizeTheme(theme);
    const textmateTheme = _vscodeTextmate.Theme.createFromRawTheme(normalizedTheme);
    themeCache.set(themeId, {
      theme: normalizedTheme,
      textmateTheme
    });
  }
  return themeCache.get(themeId);
}
async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance');
    return highlighterInstance;
  }
  try {
    log('Creating new highlighter instance...');

    // Process default theme
    const defaultTheme = getThemeContext(_githubDark.default);

    // Initialize regex engine
    const engine = (0, _engineJavascript.createJavaScriptRegexEngine)();

    // Get language registrations
    const validLanguages = Array.from(languageRegistrations.values());
    log('Loading languages:', validLanguages.map(l => l.name));

    // Create highlighter with processed components
    highlighterInstance = await (0, _core.createHighlighterCore)({
      themes: [defaultTheme.theme],
      langs: validLanguages,
      engine
    });
    const loadedLangs = highlighterInstance.getLoadedLanguages();
    log('Successfully loaded languages:', loadedLangs);
    return highlighterInstance;
  } catch (error) {
    log('Error creating highlighter:', error);
    throw error;
  }
}
function isValidLanguage(lang) {
  return languageRegistrations.has(lang.toLowerCase());
}
function getSafeLanguage(lang) {
  const normalized = lang.toLowerCase();
  const registration = languageRegistrations.get(normalized);
  if (registration) {
    return registration.name;
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`);
  return 'plaintext';
}

// For debugging
function getLoadedLanguages() {
  return Array.from(languageRegistrations.keys());
}
//# sourceMappingURL=SyntaxContext.js.map