"use strict";

/* eslint-disable no-console */
import { createHighlighterCore } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import { Theme as TextMateTheme } from '@shikijs/vscode-textmate';

// Import languages
import css from 'shiki/dist/langs/css.mjs';
import html from 'shiki/dist/langs/html.mjs';
import javascript from 'shiki/dist/langs/javascript.mjs';
import json from 'shiki/dist/langs/json.mjs';
import markdown from 'shiki/dist/langs/markdown.mjs';
import php from 'shiki/dist/langs/php.mjs';
import tsx from 'shiki/dist/langs/tsx.mjs';
import typescript from 'shiki/dist/langs/typescript.mjs';
import githubDark from 'shiki/dist/themes/github-dark.mjs';
const DEBUG = true;
function log(message, ...args) {
  if (DEBUG) {
    console.log(`[ShikiContext] ${message}`, ...args);
  }
}
// Process language grammars
function processLanguage(lang) {
  if (!lang) {
    throw new Error('Invalid language definition');
  }

  // Create language registration
  const registration = {
    name: lang.name,
    scopeName: lang.scopeName,
    aliases: lang.aliases,
    embeddedLangs: lang.embeddedLanguages,
    repository: lang.repository,
    patterns: lang.patterns
  };
  log(`Processed language ${registration.name}`, {
    name: registration.name,
    scopeName: registration.scopeName,
    aliases: registration.aliases,
    embeddedLangs: registration.embeddedLangs,
    repository: registration.repository,
    patterns: registration.patterns
  });
  return registration;
}

// Define raw languages
const rawLanguages = {
  typescript,
  javascript,
  json,
  html,
  css,
  markdown,
  tsx,
  php,
  plaintext: {
    id: 'plaintext',
    scopeName: 'source.plaintext',
    grammar: {
      patterns: [{
        match: '.*',
        name: 'text.plain'
      }]
    }
  }
};

// Process languages into proper registrations
const languageRegistrations = new Map();
for (const [key, lang] of Object.entries(rawLanguages)) {
  try {
    const registration = processLanguage(lang);
    languageRegistrations.set(key, registration);
    // Also register by scopeName for lookups
    languageRegistrations.set(registration.scopeName, registration);
    // Register aliases
    registration.aliases?.forEach(alias => {
      languageRegistrations.set(alias, registration);
    });
  } catch (error) {
    log(`Failed to process language ${key}:`, error);
  }
}
function normalizeTheme(rawTheme) {
  const theme = {
    ...rawTheme
  };
  theme.type ||= 'dark';
  theme.settings ||= [];
  theme.colorReplacements ||= {};
  if (theme.tokenColors && !theme.settings) {
    theme.settings = theme.tokenColors;
    delete theme.tokenColors;
  }
  theme.settings ||= [];
  theme.bg ||= theme.type === 'dark' ? '#1e1e1e' : '#ffffff';
  theme.fg ||= theme.type === 'dark' ? '#bbbbbb' : '#333333';
  if (!theme.settings[0] || !theme.settings[0].settings || theme.settings[0].scope) {
    theme.settings.unshift({
      settings: {
        foreground: theme.fg,
        background: theme.bg
      }
    });
  }
  return theme;
}
let highlighterInstance = null;
const themeCache = new Map();
export function getThemeContext(theme) {
  const themeId = typeof theme === 'string' ? theme : theme.name || 'default';
  if (!themeCache.has(themeId)) {
    const normalizedTheme = normalizeTheme(theme);
    const textmateTheme = TextMateTheme.createFromRawTheme(normalizedTheme);
    themeCache.set(themeId, {
      theme: normalizedTheme,
      textmateTheme
    });
  }
  return themeCache.get(themeId);
}
export async function getHighlighter() {
  if (highlighterInstance) {
    log('Returning cached highlighter instance');
    return highlighterInstance;
  }
  try {
    log('Creating new highlighter instance...');

    // Process default theme
    const defaultTheme = getThemeContext(githubDark);

    // Initialize regex engine
    const engine = createJavaScriptRegexEngine();

    // Get language registrations
    const validLanguages = Array.from(languageRegistrations.values());
    log('Loading languages:', validLanguages.map(l => l.name));

    // Create highlighter with processed components
    highlighterInstance = await createHighlighterCore({
      themes: [defaultTheme.theme],
      langs: validLanguages,
      engine
    });
    const loadedLangs = highlighterInstance.getLoadedLanguages();
    log('Successfully loaded languages:', loadedLangs);
    return highlighterInstance;
  } catch (error) {
    log('Error creating highlighter:', error);
    throw error;
  }
}
export function isValidLanguage(lang) {
  return languageRegistrations.has(lang.toLowerCase());
}
export function getSafeLanguage(lang) {
  const normalized = lang.toLowerCase();
  const registration = languageRegistrations.get(normalized);
  if (registration) {
    return registration.name;
  }
  log(`Warning: Unsupported language '${lang}', falling back to plaintext`);
  return 'plaintext';
}

// For debugging
export function getLoadedLanguages() {
  return Array.from(languageRegistrations.keys());
}
//# sourceMappingURL=SyntaxContext.js.map